﻿Public Class XISCKER_Observer
    'Byte Array in which all data received will be saved
    Dim Rx_Byte_Array(0 To 25) As Byte
    'Added to prevent threading errors during the reception of serial data
    Delegate Sub ThreadSafeCall(ByVal DataIn() As Byte)
    'Instruction Decode sub labels(RISCKER)
    Dim Opcode_R As String
    Dim Rd As String
    Dim Rs As String
    Dim Rt As String
    Dim Imm As String
    Dim Funct As String
    'CCR Binary values
    Dim CCR As String
    'Creare lists to index all Control Register(RISCKER) and CCR(CISCKER) labels
    Dim Control_Labels_list As New List(Of Label)
    'Bytes for microprocessor's Input Port value
    Dim IPBytes(0 To 1) As Byte
    'Frequency of platform operation configuration byte
    Dim Conf_Byte As Byte

    Private Sub Observer_Form_Load(sender As System.Object, e As System.EventArgs) Handles MyBase.Load
        'Show all available COM ports.
        For Each port As String In My.Computer.Ports.SerialPortNames
            COM_ComboBox.Items.Add(port)
        Next

        'Select first as default and notify if no COM Ports are available
        Try
            COM_ComboBox.SelectedItem = COM_ComboBox.Items(0)
        Catch ex As Exception
            MessageBox.Show("There are no COM Ports Available. Connect a device and refresh the COM Ports list.", "COM Port Warning", MessageBoxButtons.OK, MessageBoxIcon.Warning)
        End Try

        'Select default configuration byte
        ConfByte_ComboBox.SelectedIndex = 0

        If XISCKER_ASM.IsCISCKER Then
            '=========================CISCKER===========================
            'Set Buffer Threshold
            SerialPort.ReceivedBytesThreshold = 22
            'Fill Control Labels List with CCR Labels
            Control_Labels_list.Add(HLabel)
            Control_Labels_list.Add(ILabel)
            Control_Labels_list.Add(NLabel)
            Control_Labels_list.Add(ZLabel)
            Control_Labels_list.Add(VLabel)
            Control_Labels_list.Add(CLabel)
            '=============GUI==============
            Me.Text = "CISCKER Observer"
            'Common Resources
            ConfByte_ComboBox.Location = New Point(520, 124)
            SendBtn.Location = New Point(29, 485)
            InputPort_Txt.Location = New Point(29, 457)
            Diagram.Image = Global.XISCKER_IDE.My.Resources.CISCKER_Observer
            'Corresponding Labels
            MARLabel.Visible = True
            SP_C_Label.Visible = True
            PC_C_Label.Visible = True
            ALU_C_Label.Visible = True
            Opcode_C_Label.Visible = True
            ACCALabel.Visible = True
            ACCBLabel.Visible = True
            ACCDLabel.Visible = True
            IXLabel.Visible = True
            MMLabel.Visible = True
            Tmr1_C_Label.Visible = True
            Tmr2_C_Label.Visible = True
            OP_C_Label.Visible = True

        Else
            '=========================RISCKER===========================
            Me.Text = "RISCKER Observer"
            'Set Buffer Threshold
            SerialPort.ReceivedBytesThreshold = 26
            'Fill Control List with Control Register Labels
            Control_Labels_list.Add(TRST2)
            Control_Labels_list.Add(TRST1)
            Control_Labels_list.Add(TMRE2)
            Control_Labels_list.Add(TMRE1)
            Control_Labels_list.Add(PR2_2)
            Control_Labels_list.Add(PR2_1)
            Control_Labels_list.Add(PR1_2)
            Control_Labels_list.Add(PR1_1)
            Control_Labels_list.Add(EIE2)
            Control_Labels_list.Add(EIE1)
            Control_Labels_list.Add(TIE2)
            Control_Labels_list.Add(TIE1)
            Control_Labels_list.Add(EIRST2)
            Control_Labels_list.Add(EIRST1)
            Control_Labels_list.Add(TIRST2)
            Control_Labels_list.Add(TIRST1)
            '=============GUI==============
            'Common Resources
            ConfByte_ComboBox.Location = New Point(332, 125)
            SendBtn.Location = New Point(7, 463)
            InputPort_Txt.Location = New Point(7, 423)
            Diagram.Image = Global.XISCKER_IDE.My.Resources.RISCKER_Observer
            'Corresponding Labels
            SP_R_Label.Visible = True
            PC_R_Label.Visible = True
            ALU_R_Label.Visible = True
            InstrLabel.Visible = True
            RG0Label.Visible = True
            RG1Label.Visible = True
            RCLabel.Visible = True
            WriteDataLabel.Visible = True
            MemDataLabel.Visible = True
            Tmr1_R_Label.Visible = True
            Tmr2_R_Label.Visible = True
            OP_R_Label.Visible = True

        End If

        'Display only corresponding Labels of Architecture Control Register (RC or CCR)
        For Each Control_Label In Control_Labels_list
            Control_Label.Visible = True
        Next

    End Sub

    Private Sub RefreshCOM_Btn_Click(sender As System.Object, e As System.EventArgs) Handles RefreshCOM_Btn.Click
        'Show all available COM ports.
        COM_ComboBox.Items.Clear()

        For Each port As String In My.Computer.Ports.SerialPortNames
            COM_ComboBox.Items.Add(port)
        Next
    End Sub

    Private Sub Define_LogFile_Click(sender As Object, e As System.EventArgs) Handles Define_LogFile.Click
        'Ask where to save the .csv log file
        SaveLogDialog.Filter = "Comma Separated Values (*.csv)|*.csv"
        SaveLogDialog.DefaultExt = ".csv"
        SaveLogDialog.Title = "Log File Save Directory"
        SaveLogDialog.ShowDialog()

    End Sub

    Private Sub SaveLogDialog_FileOk(sender As Object, e As System.ComponentModel.CancelEventArgs) Handles SaveLogDialog.FileOk
        'When log file directory is chose for the first time, initialize all columns
        Try
            Using LogWriter As New IO.StreamWriter(SaveLogDialog.FileName, False)
                If XISCKER_ASM.IsCISCKER Then
                    LogWriter.Write("PC,MAR,OPCODE,MM,ALU Result,ACCA,ACCB,ACCD,IX,Half-Carry,Interrupt Mask,Negative,Zero,Overflow,Carry,SP,OP,TMR1,TMR2,IP" & vbNewLine)
                Else
                    LogWriter.Write("PC,OPCODE,RD,RS,RT,IMM,FUNCTION,ALUREG,MEMDATA,WRITEDATA,RG0,RG1,SP,OP,RC,TMR1,TMR2,IP" & vbNewLine)
                End If

            End Using
        Catch ex As Exception
            System.Media.SystemSounds.Exclamation.Play()
            MessageBox.Show("Log file (" & SaveLogDialog.FileName & ") is being used in another process.", "Log File Conflict", MessageBoxButtons.OK, MessageBoxIcon.Error)
            SaveLogDialog.FileName = ""
        End Try


    End Sub

    Private Sub ConfByte_ComboBox_SelectedIndexChanged(sender As System.Object, e As System.EventArgs) Handles ConfByte_ComboBox.SelectedIndexChanged
        Conf_Byte = ConfByte_ComboBox.SelectedIndex
        If SerialPort.IsOpen Then
            Write_SerialPort()
        End If
    End Sub

    Private Sub Connect_ToolStripBtn_Click(sender As System.Object, e As System.EventArgs) Handles Connect_ToolStripBtn.Click
        'Assign Serial Port Name and Open it
        If SerialPort.IsOpen = False Then
            If RecordPause_ToolStripBtn.Checked Then
                If SaveLogDialog.FileName = "" Then
                    MessageBox.Show("Log cannot be saved without specifying a valid Log File, disable data logging or define a valid file name.", "Log File name invalid", MessageBoxButtons.OK, MessageBoxIcon.Error)
                    Exit Sub
                End If
            End If
            If SaveLogDialog.FileName = "" Then
                RecordPause_ToolStripBtn.Enabled = False
            End If
            Try
                SerialPort.PortName = COM_ComboBox.SelectedItem
                SerialPort.BaudRate = BaudRate_ComboBox.Text
                SerialPort.Open()
            Catch ex As Exception
                System.Media.SystemSounds.Exclamation.Play()
                MessageBox.Show("Port could not be opened." & vbNewLine & "Port Name: " & COM_ComboBox.SelectedItem & _
                                vbNewLine & "Baud Rate: " & BaudRate_ComboBox.SelectedItem, "Port Error", MessageBoxButtons.OK, MessageBoxIcon.Error)
                Exit Sub
            End Try
            'Configure last system frequency selection and keep previous input port value
            Write_SerialPort()
            '"Connected" controls state
            Connect_ToolStripBtn.ToolTipText = "Disconnect"
            COM_ComboBox.Enabled = False
            RefreshCOM_Btn.Enabled = False
            BaudRate_ComboBox.Enabled = False
            InputPort_Txt.Enabled = True
            SendBtn.Enabled = True
            Define_LogFile.Enabled = False
            Connect_ToolStripBtn.Image = Global.XISCKER_IDE.My.Resources.Connect
            InputPort_Txt.Focus()
        Else
            'Same button disconnects port and sets a "Disconnected" controls state
            SerialPort.Close()
            Connect_ToolStripBtn.ToolTipText = "Connect"
            COM_ComboBox.Enabled = True
            RefreshCOM_Btn.Enabled = True
            BaudRate_ComboBox.Enabled = True
            InputPort_Txt.Enabled = False
            SendBtn.Enabled = False
            Define_LogFile.Enabled = True
            Connect_ToolStripBtn.Image = Global.XISCKER_IDE.My.Resources.Disconnect
            RecordPause_ToolStripBtn.Enabled = True
        End If
    End Sub

    Private Sub SerialPort_DataReceived(sender As Object, e As System.IO.Ports.SerialDataReceivedEventArgs) Handles SerialPort.DataReceived
        If XISCKER_ASM.IsCISCKER Then
            Do While (SerialPort.BytesToRead >= 22)
                SerialPort.Read(Rx_Byte_Array, 0, 22)
            Loop
        Else
            Do While (SerialPort.BytesToRead >= 26)
                SerialPort.Read(Rx_Byte_Array, 0, 26)
            Loop
        End If

        Received_Data(Rx_Byte_Array)

    End Sub

    Private Sub RecordPause_ToolStripBtn_Click(sender As System.Object, e As System.EventArgs) Handles RecordPause_ToolStripBtn.Click
        If RecordPause_ToolStripBtn.Checked Then
            RecordPause_ToolStripBtn.Image = Global.XISCKER_IDE.My.Resources.Pause
            RecordPause_ToolStripBtn.ToolTipText = "Pause Log"
        Else
            RecordPause_ToolStripBtn.Image = Global.XISCKER_IDE.My.Resources.Capture
            RecordPause_ToolStripBtn.ToolTipText = "Log Data"
        End If
    End Sub

    Private Sub Received_Data(ByVal DataIn() As Byte)
        ' If the calling thread is different from the thread with the control, this method creates a call-back and 
        'calls itself asynchronously using the Invoke method. From this time on, any reference to form controls are thread-safe.
        If Me.InvokeRequired Then
            Dim x As New ThreadSafeCall(AddressOf Received_Data)
            Me.Invoke(x, DataIn)
        Else
            'This are Thread-Safe Calls
            'Decode Received Data and Display it on Labels
            If XISCKER_ASM.IsCISCKER Then
                CISCKER_DecodeandDisplay()
            Else
                RISCKER_DecodeandDisplay()
            End If

            'If user checked SaveLog, a new line is logged
            If (RecordPause_ToolStripBtn.Checked = True) Then
                SaveLogFile()
            End If

        End If
    End Sub

    Private Sub RISCKER_DecodeandDisplay()
        '================Decode and Display PC State
        PC_R_Label.Text = Convert.ToUInt16(Rx_Byte_Array(1) * 256) + Convert.ToUInt16(Rx_Byte_Array(0))
        '================Decode and Display Instruction State
        Dim Instruction As String = Str2Bin(Rx_Byte_Array(5), 8, False) & Str2Bin(Rx_Byte_Array(4), 8, False) _
           & Str2Bin(Rx_Byte_Array(3), 8, False) & Str2Bin(Rx_Byte_Array(2), 8, False)

        'Obtain Opcode
        Opcode_R = Instruction.Substring(0, 4)

        'Obtain Operands
        If Opcode_R = "0001" Then
            Rd = ""
            Rs = ""
            Rt = ""
            Imm = Instruction.Substring(4, 28)
            Funct = ""
        Else
            Rd = Instruction.Substring(4, 6)
            If Opcode_R = "0111" Then
                Rs = ""
                Rt = ""
                Imm = Instruction.Substring(10, 22)
                Funct = ""
            Else
                Rs = Instruction.Substring(10, 6)
                If Opcode_R = "0000" Then
                    Rt = Instruction.Substring(16, 6)
                    Imm = ""
                    Funct = Instruction.Substring(22, 10)
                Else
                    Rt = ""
                    Imm = Instruction.Substring(16, 16)
                    Funct = ""
                End If
            End If
        End If
        'Display Instruction
        InstrLabel.Text = Opcode_R & " " & Rd & " " & Rs & " " & Rt & " " & Funct & " " & Imm
        '================Decode and Display ALUReg
        ALU_R_Label.Text = Convert.ToUInt16(Rx_Byte_Array(7) * 256) + Convert.ToUInt16(Rx_Byte_Array(6))
        '================Decode and Display MemData
        MemDataLabel.Text = Str2Hex(Rx_Byte_Array(9)) & Str2Hex(Rx_Byte_Array(8))
        '================Decode and Display WriteData
        WriteDataLabel.Text = Str2Hex(Rx_Byte_Array(11)) & Str2Hex(Rx_Byte_Array(10))
        '================Decode and Display rg0
        RG0Label.Text = Convert.ToUInt16(Rx_Byte_Array(13) * 256) + Convert.ToUInt16(Rx_Byte_Array(12))
        '================Decode and Display rg1
        RG1Label.Text = Convert.ToUInt16(Rx_Byte_Array(15) * 256) + Convert.ToUInt16(Rx_Byte_Array(14))
        '================Decode and Display sp
        SP_R_Label.Text = Convert.ToUInt16(Rx_Byte_Array(17) * 256) + Convert.ToUInt16(Rx_Byte_Array(16))
        '================Decode and Display op
        OP_R_Label.Text = Str2Hex(Rx_Byte_Array(19)) & Str2Hex(Rx_Byte_Array(18))
        '================Decode and Display rc
        RCLabel.Text = Str2Hex(Rx_Byte_Array(21)) & Str2Hex(Rx_Byte_Array(20))
        'Control Register bitwise decode to display independent bits as On or Off
        Dim Control_Register As String = Str2Bin(Rx_Byte_Array(21), 8) & Str2Bin(Rx_Byte_Array(20), 8)

        For character As Integer = 0 To 15
            If Control_Register.Chars(character) = "1" Then
                Control_Labels_list(character).ForeColor = Color.Blue
            Else
                Control_Labels_list(character).ForeColor = Color.Red
            End If

        Next

        '================Decode and Display tmr1
        Tmr1_R_Label.Text = Convert.ToUInt16(Rx_Byte_Array(23) * 256) + Convert.ToUInt16(Rx_Byte_Array(22))
        '================Decode and Display tmr2
        Tmr2_R_Label.Text = Convert.ToUInt16(Rx_Byte_Array(25) * 256) + Convert.ToUInt16(Rx_Byte_Array(24))

    End Sub

    Private Sub CISCKER_DecodeandDisplay()
        '================Decode and Display PC State
        PC_C_Label.Text = Str2Hex(Rx_Byte_Array(1)) & Str2Hex(Rx_Byte_Array(0))
        'PC_C_Label.Text = Convert.ToUInt16(Rx_Byte_Array(1) * 256) + Convert.ToUInt16(Rx_Byte_Array(0))
        '================Decode and Display Memory Address Register
        MARLabel.Text = Convert.ToUInt16(Rx_Byte_Array(3) * 256) + Convert.ToUInt16(Rx_Byte_Array(2))
        '================Decode and Display Opcode
        Opcode_C_Label.Text = Str2Hex(Rx_Byte_Array(4))
        '================Decode and Display Memory Data
        MMLabel.Text = Str2Hex(Rx_Byte_Array(6)) & Str2Hex(Rx_Byte_Array(5))
        '================Decode and Display ALU Result
        ALU_C_Label.Text = Convert.ToUInt16(Rx_Byte_Array(8) * 256) + Convert.ToUInt16(Rx_Byte_Array(7))
        '================Decode and Display Accumulator D
        ACCDLabel.Text = Convert.ToUInt16(Rx_Byte_Array(10)) + Convert.ToUInt16(Rx_Byte_Array(9) * 256)
        '================Decode and Display Accumulator A
        ACCALabel.Text = Str2Hex(Rx_Byte_Array(9))
        '================Decode and Display Accumulator B
        ACCBLabel.Text = Str2Hex(Rx_Byte_Array(10))
        '================Decode and Display Index Register
        IXLabel.Text = Convert.ToUInt16(Rx_Byte_Array(12) * 256) + Convert.ToUInt16(Rx_Byte_Array(11))
        '================Decode and Display Condition Code Register (CCR) Bitwise
        CCR = Str2Bin(Rx_Byte_Array(13), 8)

        For character As Integer = 0 To 5
            If CCR.Chars(character) = "1" Then
                Control_Labels_list(character).ForeColor = Color.Blue
            Else
                Control_Labels_list(character).ForeColor = Color.Red
            End If
        Next
        '================Decode and Display Stack Pointer
        SP_C_Label.Text = Convert.ToUInt16(Rx_Byte_Array(15) * 256) + Convert.ToUInt16(Rx_Byte_Array(14))
        '================Decode and Display Output Port
        OP_C_Label.Text = Str2Hex(Rx_Byte_Array(17)) & Str2Hex(Rx_Byte_Array(16))
        '================Decode and Display Timer 1
        Tmr1_C_Label.Text = Convert.ToUInt16(Rx_Byte_Array(19) * 256) + Convert.ToUInt16(Rx_Byte_Array(18))
        '================Decode and Display Timer 2
        Tmr2_C_Label.Text = Convert.ToUInt16(Rx_Byte_Array(21) * 256) + Convert.ToUInt16(Rx_Byte_Array(20))
    End Sub

    Private Sub SaveLogFile()
        'New log entries are appended to the existing file containing the log unless the file is being used in another process
        Try
            Using LogWriter As New IO.StreamWriter(SaveLogDialog.FileName, True)
                If XISCKER_ASM.IsCISCKER Then
                    LogWriter.WriteLine(PC_C_Label.Text & "," & MARLabel.Text & "," & Opcode_C_Label.Text & "," & MMLabel.Text & "," & _
                            ALU_C_Label.Text & "," & ACCALabel.Text & "," & ACCBLabel.Text & "," & ACCDLabel.Text & "," & IXLabel.Text & "," & _
                            CCR.Substring(0, 1) & "," & CCR.Substring(1, 1) & "," & CCR.Substring(2, 1) & "," & CCR.Substring(3, 1) & "," & CCR.Substring(4, 1) & "," & CCR.Substring(5, 1) & _
                            "," & SP_C_Label.Text & "," & OP_C_Label.Text & "," & Tmr1_C_Label.Text & "," & Tmr2_C_Label.Text & "," & InputPort_Txt.Text)
                Else
                    LogWriter.WriteLine(PC_R_Label.Text & "," & Opcode_R & "," & Rd & "," & Rs & "," & Rt & "," & Imm & "," & _
                                        Funct & "," & ALU_R_Label.Text & "," & MemDataLabel.Text & "," & WriteDataLabel.Text & "," & _
                                        RG0Label.Text & "," & RG1Label.Text & "," & SP_R_Label.Text & "," & OP_R_Label.Text & "," & RCLabel.Text & "," & _
                                        Tmr1_R_Label.Text & "," & Tmr2_R_Label.Text & "," & InputPort_Txt.Text)
                End If
                LogWriter.Flush()
                LogWriter.Close()
            End Using
        Catch ex As Exception
            System.Media.SystemSounds.Exclamation.Play()
            MessageBox.Show("Log file (" & SaveLogDialog.FileName & ") is being used in another process. Therefore the incoming states will not be logged." & _
                            vbNewLine & "Make sure to close the process using the log file before continuing", _
                            "Log File Conflict", MessageBoxButtons.OK, MessageBoxIcon.Error)
        End Try

    End Sub

    Private Sub SendBtn_Click(sender As System.Object, e As System.EventArgs) Handles SendBtn.Click
        Try
            IPBytes = BitConverter.GetBytes(NumericalBase(InputPort_Txt.Text))

        Catch ex As Exception
            MessageBox.Show("The value corresponding to the Input Port is not valid, change it to a 16bit formatted word." & vbNewLine & _
                            "i.e FFFFh, 111111111111111b, 177777o or 65535", "Input port data not valid", MessageBoxButtons.OK, MessageBoxIcon.Error)
        End Try

        Write_SerialPort()

    End Sub

    Private Sub Write_SerialPort()

        Try
            SerialPort.Write(IPBytes, 0, 1)
            SerialPort.Write(IPBytes, 1, 1)
            SerialPort.Write(Conf_Byte)

        Catch ex As Exception
            System.Media.SystemSounds.Exclamation.Play()
            MessageBox.Show("Data could not be sent. Verify the system connection and data format", "Port write error", MessageBoxButtons.OK, MessageBoxIcon.Error)
        End Try

    End Sub

    Private Function Str2Hex(ByVal strByte As Byte) As String
        Str2Hex = UCase(Convert.ToString(strByte, 16))
        If Str2Hex.Length = 1 Then
            Str2Hex = "0" & Str2Hex
        End If
    End Function

    Private Sub XISCKER_Observer_FormClosing(sender As Object, e As System.Windows.Forms.FormClosingEventArgs) Handles Me.FormClosing
        If SerialPort.IsOpen Then
            SerialPort.Close()
        End If

        Control_Labels_list.Clear()
        IPBytes = {0, 0}

    End Sub

    Public Function Str2Bin(ByVal strOP As String, Optional ByVal NumOfBits As Byte = 16, Optional ByVal Signed As Boolean = False) As String

        Dim j As Double
        Dim intOP As Integer
        Dim MaxBitRepr As Byte


        If Signed = True Then
            Try
                intOP = CInt(strOP)
                MaxBitRepr = NumOfBits - 1
            Catch ex As Exception
                Str2Bin = "%"
                Exit Function
            End Try

        Else
            Try
                intOP = Convert.ToUInt32(strOP)
                MaxBitRepr = NumOfBits
            Catch ex As Exception
                Str2Bin = "%"
                Exit Function
            End Try

        End If

        'Check for overload errors
        Str2Bin = ""
        If intOP < (-2 ^ MaxBitRepr) Then
            Str2Bin = "min"
            Exit Function

        ElseIf intOP > ((2 ^ MaxBitRepr) - 1) Then
            Str2Bin = "max"
            Exit Function

            'check for sign
        ElseIf Signed = True And intOP < 0 Then
            Str2Bin = "1"
            intOP = intOP + 2 ^ (MaxBitRepr)
            j = 2 ^ (MaxBitRepr - 1)
        Else
            j = 2 ^ (NumOfBits - 1)
        End If

        'Binary conversion
        Do While j >= 1
            Str2Bin = Str2Bin & Fix(intOP / j)
            intOP = intOP - j * Fix(intOP / j)
            j = j / 2
        Loop

        'Check the width
        If Str2Bin.Length < NumOfBits Then
            Str2Bin = "0" & Str2Bin
        End If

    End Function

    Public Function NumericalBase(ByVal Number As String) As UInt16
        If Number.Last = "h" Then
            Number = Replace(Number, Number.Last, "")
            NumericalBase = Convert.ToUInt16(Number, 16)
        ElseIf Number.Last = "b" Then
            Number = Replace(Number, Number.Last, "")
            NumericalBase = Convert.ToUInt16(Number, 2)
        ElseIf Number.Last = "o" Then
            Number = Replace(Number, Number.Last, "")
            NumericalBase = Convert.ToUInt16(Number, 8)
        Else
            NumericalBase = Convert.ToUInt16(Number, 10)
        End If
    End Function

End Class
